#ifndef MUNIXAPPLICATIONICON_H
#define MUNIXAPPLICATIONICON_H

#include <QObject>
#include <QScopedPointer>

/*!
 * \brief The MUnixApplicationIcon class
 *
 * This class can create and remove an application ico entry for the application
 * based on its QCoreApplication::applicationName()
 */
class MUnixApplicationIcon : public QObject
{
    Q_OBJECT
    Q_PROPERTY(bool isEnabled READ isEnabled WRITE setEnabled NOTIFY isEnabledChanged)

public:
    /*!
     * \brief MUnixApplicationIcon constructor
     *
     * Set the QCoreApplication::applicationName before instantiation
     */
    MUnixApplicationIcon(QObject *parent = nullptr);
    ~MUnixApplicationIcon();

signals:
    void isEnabledChanged(bool enabled);

public slots:
    /*!
     * \brief isGLoballyEnabled
     * \return Returns whether an application icon is globally enabled (installed in /usr)
     *
     * \code
     * //You have to set the application name before you use this class
     * QCoreApplication::setApplicationName(QLatin1String("My amazing app"));
     *
     * // Is the application icon insatlled?
     * qDebug() << MUnixApplicationIcon().isGLoballyEnabled();
     * \endcode
     */
    bool isGLoballyEnabled() const;

    /*!
     * \brief isEnabled
     * \return Returns whether an application icon for the application extsts or not
     *
     * \code
     * //You have to set the application name before you use this class
     * QCoreApplication::setApplicationName(QLatin1String("My amazing app"));
     *
     * // Is the application icon insatlled?
     * qDebug() << MUnixApplicationIcon().isEnabled();
     * \endcode
     */
    bool isEnabled() const;

    /*!
     * \brief setEnabled
     * \param whether you want to enable or disable the application icon
     *
     * \code
     * //You have to set the application name before you use this class
     * QCoreApplication::setApplicationName(QLatin1String("My amazing app"));
     *
     * // Create an autostart entry for the application
     * MUnixApplicationIcon().setEnabled(true);
     * \endcode
     */
    void setEnabled(bool enabled);

private:
    class Private;
    QScopedPointer<Private> priv;
};

#endif // MUNIXAPPLICATIONICON_H
