#include "munixdesktopenvironment.h"

//! \cond DOXYGEN_EXCLUDE
struct MUnixDesktopEnvironmentPrivate
{
    static MUnixDesktopEnvironment::DesktopEnvironment detectedEnvironment;
    static bool alreadyDetected;
};
//! \endcond DOXYGEN_EXCLUDE

MUnixDesktopEnvironment::DesktopEnvironment MUnixDesktopEnvironmentPrivate::detectedEnvironment = MUnixDesktopEnvironment::DesktopEnvironment_Other;
bool MUnixDesktopEnvironmentPrivate::alreadyDetected = false;


MUnixDesktopEnvironment::DesktopEnvironment getKdeVersion() {
    QByteArray value = qgetenv("KDE_SESSION_VERSION");
    if ( value == "5" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Plasma5;
    } else if (value == "4" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Kde4;
    } else {
        // most likely KDE3
        return MUnixDesktopEnvironment::DesktopEnvironment_Other;
    }
}

MUnixDesktopEnvironment::DesktopEnvironment detectDesktopEnvironment() {
    QByteArray xdgCurrentDesktop = qgetenv("XDG_CURRENT_DESKTOP");
    if ( xdgCurrentDesktop == "GNOME" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Gnome;
    } else if ( xdgCurrentDesktop == "Unity" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Unity;
    } else if ( xdgCurrentDesktop == "KDE" ) {
        return getKdeVersion();
    }

    QByteArray desktopSession = qgetenv("DESKTOP_SESSION");
    if ( desktopSession == "gnome" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Gnome;
    } else if ( desktopSession == "kde" ) {
        return getKdeVersion();
    } else if ( desktopSession == "kde4" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Kde4;
    } else if ( desktopSession.contains("xfce") || desktopSession == "xubuntu" ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Xfce;
    }

    if ( !qgetenv("GNOME_DESKTOP_SESSION_ID").isEmpty() ) {
        return MUnixDesktopEnvironment::DesktopEnvironment_Gnome;
    } else if ( !qgetenv("KDE_FULL_SESSION").isEmpty() ) {
        return getKdeVersion();
    }

    return MUnixDesktopEnvironment::DesktopEnvironment_Other;
}

MUnixDesktopEnvironment::DesktopEnvironment MUnixDesktopEnvironment::detect()
{
    if (!MUnixDesktopEnvironmentPrivate::alreadyDetected) {
        MUnixDesktopEnvironmentPrivate::detectedEnvironment = detectDesktopEnvironment();
        MUnixDesktopEnvironmentPrivate::alreadyDetected = true;
    }

    return MUnixDesktopEnvironmentPrivate::detectedEnvironment;
}
