#include "checkboxitemdelegate.h"
#include <QApplication>
#include <QPainter>
#include <QDebug>
#include <QMouseEvent>

CheckBoxItemDelegate::CheckBoxItemDelegate(QObject *parent) : QStyledItemDelegate(parent)
{

}


void CheckBoxItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{

    QStyleOptionViewItem opt = option;
    //initStyleOption(&opt, index);
    opt.state &= ~QStyle::State_HasFocus;
    if (index.flags() & Qt::ItemIsUserCheckable) {
        opt.state |= QStyle::State_Enabled;
    } else {
        opt.state &= ~QStyle::State_Enabled;
    }
    QApplication::style()->drawControl(QStyle::CE_ItemViewItem, &opt, painter);


    QStyleOptionButton styleOption;
    styleOption.state |= index.data(Qt::CheckStateRole).toBool() ? QStyle::State_On : QStyle::State_Off;

    Qt::ItemFlags flags = index.model()->flags(index);
    if (!(flags & Qt::ItemIsUserCheckable) || !(flags & Qt::ItemIsEnabled)) {
        styleOption.state |=  QStyle::QStyle::State_Sunken;
    }

    QSize checkBoxsize = QApplication::style()->sizeFromContents(QStyle::CT_CheckBox, &styleOption, QSize());
    // center
    styleOption.rect = QRect(option.rect.x() + (option.rect.width()/2 - checkBoxsize.width()/2),
                             option.rect.y() + (option.rect.height()/2 - checkBoxsize.height()/2),
                             checkBoxsize.width(),
                             checkBoxsize.height());

    QApplication::style()->drawControl(QStyle::CE_CheckBox, &styleOption, painter);
}

QSize CheckBoxItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(option) Q_UNUSED(index)
    QStyleOptionButton styleOption;
    QSize size = QApplication::style()->sizeFromContents(QStyle::CT_CheckBox, &styleOption, QSize());
    return size;
}


bool CheckBoxItemDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option,
                         const QModelIndex &index)
{
    Q_ASSERT(event);
    Q_ASSERT(model);

    // make sure that the item is checkable
    Qt::ItemFlags flags = model->flags(index);
    if (!(flags & Qt::ItemIsUserCheckable) || !(flags & Qt::ItemIsEnabled)) {
        return false;
    }

    // make sure that we have a check state
    QVariant value = index.data(Qt::CheckStateRole);
    if (!value.isValid()) {
        return false;
    }

    // make sure that we have the right event type
    if (event->type() == QEvent::MouseButtonRelease) {
        QStyleOptionButton styleOption;
        QSize checkBoxsize = QApplication::style()->sizeFromContents(QStyle::CT_CheckBox, &styleOption, QSize());
        QRect rect = QRect(option.rect.x() + (option.rect.width()/2 - checkBoxsize.width()/2),
                           option.rect.y() + (option.rect.height()/2 - checkBoxsize.height()/2),
                           checkBoxsize.width(),
                           checkBoxsize.height());

        if (rect.contains(dynamic_cast<QMouseEvent*>(event)->pos())) {
            Qt::CheckState state = (static_cast<Qt::CheckState>(value.toInt()) == Qt::Checked
                                    ? Qt::Unchecked : Qt::Checked);

            emit toggled(index, !value.toBool());
            return model->setData(index, state, Qt::CheckStateRole);
        }
    }
    return false;
}
