#ifndef ADDSTASHPAGE_H
#define ADDSTASHPAGE_H

#include <QSharedPointer>
#include <QWizardPage>

class Stash;

namespace Ui {
class AddStashPage;
}

class AddStashPage : public QWizardPage
{
    Q_OBJECT

public:
    explicit AddStashPage(const QSharedPointer<Stash> &stash, QWidget *parent = nullptr);
    ~AddStashPage();

private:
    QString stashDir() const;
    QString mountDir() const;
    QString password() const;

public slots:
    virtual void setVisible(bool visible) override;
    void createResult(bool success, const QString &message);

private slots:
    void stashIsBusyChanged(bool busy);
    void stashDirChanged(const QString &stashDir);
    void mountDirChanged(const QString &mountDir);
    void init();
    void setMessage(const QString &message);
    void adjustDialogSize();
    bool validate();
    void generateDefaultStashDir();
    void generateDefaultMountDir();
    void on_label_help_linkActivated(const QString &link);
    void on_button_add_clicked();
    void on_button_cancel_clicked();
    void on_lineEdit_stashDir_returnPressed();
    void on_lineEdit_mountDir_returnPressed();
    void on_lineEdit_password_returnPressed();
    void on_lineEdit_retypePassword_returnPressed();
    void on_button_stashDir_clicked();
    void on_button_mountDir_clicked();

private:
    Ui::AddStashPage *ui;
    QSharedPointer<Stash> m_stash;
    bool m_initialized;
};

#endif // ADDSTASHPAGE_H
