#include "changepasswordpage.h"
#include "ui_changepasswordpage.h"

// Qt
#include <QDebug>
#include <QIcon>

// Core
#include "Core/settings.h"
#include "Core/stash.h"
#include "Core/stashmanager.h"

#include <MHelpBrowser>

ChangePasswordPage::ChangePasswordPage(const QSharedPointer<Stash> &stash, QWidget *parent) :
    QWizardPage(parent),
    ui(new Ui::ChangePasswordPage),
    m_stash(stash),
    m_initialized(false),
    m_size(QSize(1,1))
{
    ui->setupUi(this);

    setPixmap(QWizard::LogoPixmap, QIcon::fromTheme(QLatin1String("document-edit")).pixmap(24,24));

    ui->label_help->setText(QString(QLatin1String("<html><a href=\"stashchangepassword.html\">%1</a></html>")).arg(tr("Help")));

    ui->button_sidebar->setChecked(Settings::instance().showSidebar());
    connect(ui->button_sidebar, &QPushButton::toggled, &Settings::instance(), &Settings::setShowSidebar);
    connect(&Settings::instance(), &Settings::showSidebarChanged, ui->button_sidebar, &QPushButton::setChecked);

    connect(ui->lineEdit_oldPassword, &QLineEdit::textChanged, this, &ChangePasswordPage::validate);
    connect(ui->lineEdit_newPassword, &QLineEdit::textChanged, this, &ChangePasswordPage::validate);
    connect(ui->lineEdit_retypeNewPassword, &QLineEdit::textChanged, this, &ChangePasswordPage::validate);

    connect(m_stash.data(), &Stash::isBusyChanged, ui->button_change, &MSpinnerPushButton::setChecked);
    connect(m_stash.data(), &Stash::isBusyChanged, ui->button_change, &MSpinnerPushButton::setDisabled);

    connect(m_stash.data(), &Stash::changePasswordFinished, this, &ChangePasswordPage::changePasswordResult);

    int labelWidth = qMax(ui->label_oldPassword->sizeHint().width(), ui->label_newPassword->sizeHint().width());
    labelWidth = qMax(labelWidth, ui->label_retypeNewPassword->sizeHint().width());
    ui->label_oldPassword->setMinimumWidth(labelWidth);
    ui->label_newPassword->setMinimumWidth(labelWidth);
    ui->label_retypeNewPassword->setMinimumWidth(labelWidth);

    validate();
}

ChangePasswordPage::~ChangePasswordPage()
{
    delete ui;
}

void ChangePasswordPage::setVisible(bool visible)
{
    if (visible && !m_initialized) {
        m_initialized = true;

    }

    if (visible) {
        m_size = QWizardPage::minimumSizeHint();
    } else {
        m_size = QSize(1,1);
    }

    QWizardPage::setVisible(visible);

    if (visible) {
        ui->lineEdit_oldPassword->setFocus();
    }
}

QSize ChangePasswordPage::sizeHint() const
{
    return m_size;
}

QSize ChangePasswordPage::minimumSizeHint() const
{
    return m_size;
}

void ChangePasswordPage::changePasswordResult(bool success, const QString &message)
{
    if (success) {
        qobject_cast<QDialog*>(parent()->parent())->close();
    } else {
        setMessage(message);
    }
}

void ChangePasswordPage::validate()
{
    if (ui->lineEdit_oldPassword->text().isEmpty()) {
        setMessage(tr("Please enter the old password"));
        ui->button_change->setEnabled(false);
        return;
    }

    if (ui->lineEdit_newPassword->text().isEmpty()) {
        setMessage(tr("Please enter the new password"));
        ui->button_change->setEnabled(false);
        return;
    }

    if (ui->lineEdit_retypeNewPassword->text().isEmpty()) {
        setMessage(tr("Please retype the new password"));
        ui->button_change->setEnabled(false);
        return;
    }

    if (ui->lineEdit_newPassword->text() != ui->lineEdit_retypeNewPassword->text()) {
        setMessage(tr("Passwords do not match"));
        ui->button_change->setEnabled(false);
        return;
    }

    setMessage(QLatin1String());
    ui->button_change->setEnabled(true);
}

void ChangePasswordPage::setMessage(const QString &message)
{
    if (message.isEmpty()) {
        ui->label_message->hide();
    } else {
        ui->label_message->setText(QStringLiteral("<span style=\"color:#ff0000;\">✘</span> ") + message);
        ui->label_message->show();
    }
    //adjustDialogSize();
}

void ChangePasswordPage::on_button_cancel_clicked()
{
    if (m_stash->isBusy()) {
        StashManager::instance().cancel(m_stash);
    } else {
        qobject_cast<QDialog*>(parent()->parent())->reject();
    }
}

void ChangePasswordPage::on_button_change_clicked()
{
    StashManager::instance().changePassword(m_stash, ui->lineEdit_oldPassword->text(), ui->lineEdit_newPassword->text());
}

void ChangePasswordPage::on_lineEdit_oldPassword_returnPressed()
{
    ui->button_change->click();
}

void ChangePasswordPage::on_lineEdit_newPassword_returnPressed()
{
    ui->button_change->click();
}

void ChangePasswordPage::on_lineEdit_retypeNewPassword_returnPressed()
{
    ui->button_change->click();
}

void ChangePasswordPage::on_label_help_linkActivated(const QString &link)
{
    MHelpBrowser::open(link);
}
